%{
/*
 * Copyright (c) 2021 Mellanox Technologies LTD. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#include <opensm/osm_file_ids.h>
#define FILE_ID OSM_FILE_DEVICE_CONF_PARSER_L_L
#include "osm_device_conf_parser_y.h"

extern char *__device_conf_parser_yylval;
extern char *trim_whitespace(char *str);
#define SAVE_POS save_pos()
static void save_pos();

extern int cap_column_num;
extern int cap_line_num;
%}

%option nounput noinput noyywrap
STRING [a-zA-Z0-9'.''_''\-''|''&''@''('')''/''#''!''?''+''=''[''\]''\/''{''}'';''%''*''^''\\'':']*

%%

\#.*\n			{ SAVE_POS; }
[ \t]*\n		{ SAVE_POS; } /* ignore end of line and empty lines*/
[' ']*			{ SAVE_POS; } /* ignore spaces */
,			{ SAVE_POS; return COMMA; }
true			{ SAVE_POS; return TK_TRUE; }
false			{ SAVE_POS; return TK_FALSE; }

[ \t]*port\-conf[ \t]* 	{ SAVE_POS; return PORT_CONF_START; }
[ \t]*end\-port\-conf[ \t]*    { SAVE_POS; return PORT_CONF_END; }
[ \t]*port\-group\-name:[ \t]* 	{ SAVE_POS; return PORT_GROUP_NAME; }
[ \t]*max\-op\-vl:[ \t]*	{ SAVE_POS; return GROUP_MAX_OP_VL; }
[ \t]*sharp\-enabled:[ \t]* { SAVE_POS; return GROUP_SHARP_ENABLED; }
[ \t]*hoq\-life:[ \t]*		{ SAVE_POS; return GROUP_HOQ_LIFE; }
[ \t]*qos\-config\-vl\-start[ \t]*	{ SAVE_POS; return QOS_CONFIG_VL_START; }
[ \t]*qos\-config\-vl\-end[ \t]*	{ SAVE_POS; return QOS_CONFIG_VL_END; }
[ \t]*vl\-buffer\-weight:[ \t]*		{ SAVE_POS; return VL_BUFFER_WEIGHT; }
[ \t]*data\-type\-bitmask:[ \t]*	{ SAVE_POS; return DATA_TYPE_BITMASK; }
[ \t]*disable\-hoq\-life:[ \t]*		{ SAVE_POS; return DISABLE_HOQ_LIFE; }

0[xX][0-9a-fA-F]+ {
			SAVE_POS;
			__device_conf_parser_yylval = strdup(yytext);
			return NUMBER;
                  }

[0-9]+		  {
			SAVE_POS;
			free(__device_conf_parser_yylval);
			__device_conf_parser_yylval = strdup(yytext);
			return NUMBER;
                  }
{STRING} {
		free(__device_conf_parser_yylval);
		__device_conf_parser_yylval=strdup(trim_whitespace(yytext));
		SAVE_POS;
		return TEXT;
	}
%%

static void save_pos()
{
	int i;
	for (i = 0; i < yyleng; i++) {
		if (yytext[i] == '\n') {
			cap_line_num++;
			cap_column_num = 1;
		} else {
			cap_column_num++;
		}
	}
}
