%{
/*
 * Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#include <opensm/osm_file_ids.h>
#define FILE_ID OSM_FILE_FAST_RECOVERY_PARSER_L_L
#include "osm_fast_recovery_parser_y.h"

extern char *__fast_recovery_parser_yylval;
extern char *trim_whitespace(char *str);
#define SAVE_POS save_pos()
static void save_pos();

extern int fast_recovery_column_num;
extern int fast_recovery_line_num;
%}

%option nounput noinput noyywrap
STRING [a-zA-Z0-9'_''|''&''@''('')''/''#''!''?''+''=''[''\]''\/''{''}'';''%''*''^''\\'':']*

%%

\#.*\n			{ SAVE_POS; return COMMENT; }
[ \t]*\n		{ SAVE_POS; } /* ignore end of line and empty lines*/
[' ']*			{ SAVE_POS; } /* ignore spaces */
,			{ SAVE_POS; return COMMA; }
true			{ SAVE_POS; return TK_TRUE; }
false			{ SAVE_POS; return TK_FALSE; }

[ \t]*port\_conf\_start[ \t]* 		{ SAVE_POS; return PORT_CONF_START; }
[ \t]*port\_conf\_end[ \t]*    		{ SAVE_POS; return PORT_CONF_END; }
[ \t]*port\_group\_name:[ \t]* 		{ SAVE_POS; return PORT_GROUP_NAME; }

[ \t]*credit\_watchdog\_start[ \t]*	{ SAVE_POS; return CREDIT_WATCHDOG_START; }
[ \t]*credit\_watchdog\_end[ \t]*	{ SAVE_POS; return CREDIT_WATCHDOG_END; }
[ \t]*raw\_ber\_start[ \t]*		{ SAVE_POS; return RAW_BER_START; }
[ \t]*raw\_ber\_end[ \t]*		{ SAVE_POS; return RAW_BER_END; }
[ \t]*effective\_ber\_start[ \t]*	{ SAVE_POS; return EFFECTIVE_BER_START; }
[ \t]*effective\_ber\_end[ \t]*		{ SAVE_POS; return EFFECTIVE_BER_END; }
[ \t]*symbol\_ber\_start[ \t]*		{ SAVE_POS; return SYMBOL_BER_START; }
[ \t]*symbol\_ber\_end[ \t]*		{ SAVE_POS; return SYMBOL_BER_END; }

[ \t]*error\_thr\_action:[ \t]*		{ SAVE_POS; return GROUP_ERR_THR_ACT; }
[ \t]*no\_reset[ \t]*			{ SAVE_POS; return ERR_ACT_NO_RESET; }
[ \t]*physical\_reset[ \t]*		{ SAVE_POS; return ERR_ACT_PHYSICAL_RESET; }
[ \t]*logical\_reset[ \t]*		{ SAVE_POS; return ERR_ACT_LOGICAL_RESET; }

[ \t]*enable\_threshold:[ \t]*		{ SAVE_POS; return GROUP_ENABLE_THR; }
[ \t]*none[ \t]*			{ SAVE_POS; return ENABLE_NONE; }
[ \t]*error[ \t]*			{ SAVE_POS; return ENABLE_ERROR; }
[ \t]*warning\_normal[ \t]*		{ SAVE_POS; return ENABLE_WARN_NORMAL; }
[ \t]*all[ \t]*				{ SAVE_POS; return ENABLE_ALL; }

[ \t]*error\_threshold:[ \t]*		{ SAVE_POS; return GROUP_ERR_THR; }
[ \t]*warning\_threshold:[ \t]*		{ SAVE_POS; return GROUP_WARN_THR; }
[ \t]*normal\_threshold:[ \t]*		{ SAVE_POS; return GROUP_NORM_THR; }

[ \t]*time\_window:[ \t]*		{ SAVE_POS; return GROUP_TIME_WINDOW; }
[ \t]*sampling\_rate:[ \t]*		{ SAVE_POS; return GROUP_SAMPLING_RATE; }

[ \t]*apply\_thresholds:[ \t]*		{ SAVE_POS; return GROUP_DEFAULT_THR; }
[ \t]*fw\_default[ \t]*			{ SAVE_POS; return TK_TRUE; }
[ \t]*sm\_config[ \t]*			{ SAVE_POS; return TK_FALSE; }


0[xX][0-9a-fA-F]+ {
			SAVE_POS;
			__fast_recovery_parser_yylval = strdup(yytext);
			return NUMBER;
                  }

[0-9]+e\-[0-9]+ |
[0-9]+\.[0-9]+e\-[0-9]+ |
[0-9]+\.[0-9]+	  {
			free(__fast_recovery_parser_yylval);
			__fast_recovery_parser_yylval=strdup(trim_whitespace(yytext));
			SAVE_POS;
			return NUMBER_EXP;
                  }

[0-9]+[^\.]	{
			SAVE_POS;
			free(__fast_recovery_parser_yylval);
			__fast_recovery_parser_yylval = strdup(yytext);
			return NUMBER;
                  }
{STRING} {
		free(__fast_recovery_parser_yylval);
		__fast_recovery_parser_yylval=strdup(trim_whitespace(yytext));
		SAVE_POS;
		return TEXT;
	}
%%

static void save_pos()
{
	int i;
	for (i = 0; i < yyleng; i++) {
		if (yytext[i] == '\n') {
			fast_recovery_line_num++;
			fast_recovery_column_num = 1;
		} else {
			fast_recovery_column_num++;
		}
	}
}
